------------------------------------------------------------------------------
--                                                                          --
--                         GNAT COMPILER COMPONENTS                         --
--                                                                          --
--                             E X P _ I N T R                              --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                            $Revision: 1.3 $                              --
--                                                                          --
--           Copyright (c) 1992,1993,1994 NYU, All Rights Reserved          --
--                                                                          --
-- GNAT is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  GNAT is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with GNAT;  see file COPYING.  If not, write --
-- to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. --
--                                                                          --
------------------------------------------------------------------------------

with Atree;    use Atree;
with Einfo;    use Einfo;
with Exp_Util; use Exp_Util;
with Namet;    use Namet;
with Nmake;    use Nmake;
with Nlists;   use Nlists;
with Output;   use Output;
with Rtsfind;  use Rtsfind;
with Sem;      use Sem;
with Sem_Util; use Sem_Util;
with Sinfo;    use Sinfo;
with Snames;   use Snames;
with Tbuild;   use TBuild;

package body Exp_Intr is

   -----------------------
   -- Local Subprograms --
   -----------------------

   procedure Expand_Shift (N : Node_Id; E : Entity_Id; K : Node_Kind);
   --  Expand an intrinsic shift operation, N and E are from the call to
   --  Expand_Instrinsic_Call (call node and subprogram spec entity) and
   --  K is the kind for the shift node

   procedure Expand_Unc_Conversion (N : Node_Id; E : Entity_Id);
   --  Expand a call to an instantiation of Unchecked_Convertion into a node
   --  N_Unchecked_Type_Conversion.

   procedure Expand_Unc_Deallocation (N : Node_Id; E : Entity_Id);
   --  Expand a call to an instantiation of Unchecked_Deallocation into a node
   --  N_Free_Statement and appropriate context.


   ----------------------------
   -- Expand_Instrinsic_Call --
   ----------------------------

   procedure Expand_Intrinsic_Call (N : Node_Id; E : Entity_Id) is
      Nam : Name_Id;

   begin

      --  If the intrinsic subprogram is generic, gets its original name.

      if Present (Parent (E))
        and then Present (Generic_Parent (Parent (E)))
      then
         Nam := Chars (Generic_Parent (Parent (E)));
      else
         Nam := Chars (E);
      end if;

      if Nam = Name_Rotate_Left then
         Expand_Shift (N, E, N_Op_Rotate_Left);
      elsif Nam = Name_Rotate_Right then
         Expand_Shift (N, E, N_Op_Rotate_Right);
      elsif Nam = Name_Shift_Left then
         Expand_Shift (N, E, N_Op_Shift_Left);
      elsif Nam = Name_Shift_Right then
         Expand_Shift (N, E, N_Op_Shift_Right);
      elsif Nam = Name_Shift_Right_Arithmetic then
         Expand_Shift (N, E, N_Op_Shift_Right_Arithmetic);
      elsif Nam = Name_Unchecked_Conversion then
         Expand_Unc_Conversion (N, E);
      elsif Nam = Name_Unchecked_Deallocation then
         Expand_Unc_Deallocation (N, E);
      else
         pragma Assert (False); null;
      end if;
   end Expand_Intrinsic_Call;

   ------------------
   -- Expand_Shift --
   ------------------

   procedure Expand_Shift (N : Node_Id; E : Entity_Id; K : Node_Kind) is
      Loc   : constant Source_Ptr := Sloc (N);
      Typ   : constant Entity_Id  := Etype (N);
      Left  : constant Node_Id    := First_Actual (N);
      Right : constant Node_Id    := Next_Actual (Left);
      Snode : Node_Id;

   begin
      Snode := New_Node (K, Loc);
      Set_Left_Opnd  (Snode, Relocate_Node (Left));
      Set_Right_Opnd (Snode, Relocate_Node (Right));
      Set_Chars      (Snode, Chars (E));
      Set_Etype      (Snode, Typ);
      Set_Entity     (Snode, E);

      --  Do the replacement. Note that we don't call Analyze and Resolve
      --  on this node, because it already got analyzed and resolved when
      --  it was a function call!

      Replace_Substitute_Tree (N, Snode);
      Set_Analyzed (N);

   end Expand_Shift;

   ---------------------------
   -- Expand_Unc_Conversion --
   ---------------------------

   procedure Expand_Unc_Conversion (N : Node_Id; E : Entity_Id) is
      Loc   : constant Source_Ptr := Sloc (N);

   begin
      Replace_Substitute_Tree (N,
        Make_Unchecked_Type_Conversion (Loc,
          Subtype_Mark => New_Occurrence_Of (Etype (E), Loc),
          Expression   => Relocate_Node (First_Actual (N))));

      Set_Etype (N, Etype (E));
      Set_Analyzed (N);
   end Expand_Unc_Conversion;

   -----------------------------
   -- Expand_Unc_Deallocation --
   -----------------------------

   --  Generate the following Code :
   --    if Arg /= null then
   --       <if Has_Controlled (Designated_Type (Typ)) then>
   --          Deep_Finalize (Arg.all);
   --       <elsif Is_Controlled (Designated_Type (Typ)) then>
   --          Finalize (Arg.all);
   --       Free (Arg);
   --       Arg := Null;
   --    end if;

   procedure Expand_Unc_Deallocation (N : Node_Id; E : Entity_Id) is
      Loc       : constant Source_Ptr := Sloc (N);
      Arg       : constant Node_Id := Relocate_Node (First_Actual (N));
      Typ       : constant Entity_Id := Etype (Arg);
      Stmts     : constant List_Id := New_List;
      Free_Node : Node_Id;

   begin
      Set_Evaluate_Once (Arg);

      if Has_Controlled (Designated_Type (Typ)) then

         null;  --  Not Implemented Yet ???

      elsif Is_Controlled (Designated_Type (Typ)) then

         --  Generate the Finalization call

         Append_To (Stmts,
           Make_Procedure_Call_Statement (Loc,
             Name => New_Reference_To (RTE (RE_Finalize_One), Loc),
             Parameter_Associations => New_List (
               New_Reference_To (
                 Finalization_Chain_Entity (Scope (Typ)), Loc),

               Make_Unchecked_Type_Conversion (Loc,
                 Subtype_Mark =>
                   New_Reference_To (RTE (RE_Finalizable), Loc),
                 Expression => Arg))));
      end if;

      Free_Node := Make_Free_Statement (Loc, Expression => Arg);
      Set_Storage_Pool (Free_Node, Associated_Storage_Pool (Etype (Arg)));

      if Present (Storage_Pool (Free_Node)) then
         Set_Procedure_To_Call (Free_Node,
           Find_Prim_Op (Etype (Storage_Pool (Free_Node)), Name_Deallocate));
      end if;

      Set_Analyzed (Free_Node);

      Append_To (Stmts, Free_Node);

      Append_To (Stmts,
        Make_Assignment_Statement (Loc,
          Name => Arg,
          Expression => Make_Null (Loc)));

      Replace_Substitute_Tree (N,
        Make_If_Statement (Loc,
          Condition =>
            Make_Op_Ne (Loc,
              Left_Opnd  => Arg,
              Right_Opnd => Make_Null (Loc)),

          Then_Statements => Stmts));

      Analyze (N);
   end Expand_Unc_Deallocation;

end Exp_Intr;
